%   This program generates draws from the multinomial probability with linear covarites
%       for the clustering program of Hamilton and Owyang
%   The posterior probability is:
%       Pr[Hprime(n,k) = 1|Y,H(~n)] = p(Y|H(n),TH(n)) x p(H(n)|H(~n))
%   ********************************************************************************************************
%   The first part of the program generates the betas, conditional on H(~n)
%   Then, we update the posterior cluster probability using multinomial probability
%   Then, we can draw the clustering based on the posterior probability

function [H,KAY,bta,lam,z] = HO_H_CV(priors,draw)

KAY = zeros(priors.N,priors.no_clust);
%    Draw the parameters for the logit prior
[bta,lam,z] = HO_MNL_CV(priors,draw);
accept = 0;count = 0;
while accept == 0
    for n = 1:priors.N
        %   p(Y|H(n,k)=1,THETA(n),H(~n),H(n,~k))
        for k = 1:priors.no_clust
            %   compute the probabilities Pr[Q_{nj}=1|x_{n},?]
            %   = exp(bx) / sum(exp(bx))
            pr1 = exp(priors.x(n,:) * draw.beta(:,k)) / (exp(priors.x(n,:) * draw.beta(:,k)) + 1);
            pr0 = 1 / (exp(priors.x(n,:) * draw.beta(:,k)) + 1);
            %   the counterfactual state processes
            sk0 = draw.Sk(:,2);%zeros(priors.T,1);
            sk1 = draw.Sk(:,2);%zeros(priors.T,1);
            k0 = draw.KAY(n,:);
            k0(:,k) = 0;
            k1 = draw.KAY(n,:);
            k1(:,k) = 1;
            for i = 3:priors.K
                sk0 = sk0 + k0(1,i-2) * draw.Sk(:,i);
                sk1 = sk1 + k1(1,i-2) * draw.Sk(:,i);
            end,
            %   compute the forecast error for each cluster possibility
            ferr0 = priors.YY(:,n) - draw.mu_0(n,:) - draw.mu_1(n,:) * sk0;
            ferr1 = priors.YY(:,n) - draw.mu_0(n,:) - draw.mu_1(n,:) * sk1;
            log_lk0 = - (ferr0' * ferr0) / (2 * draw.sigsq(n,:));
            log_lk1 = - (ferr1' * ferr1) / (2 * draw.sigsq(n,:));
            %   for over/underrun
            scale = min([- log_lk0, -log_lk1]) - 1;
            lk0 = exp(log_lk0 + scale * ones(size(log_lk0)));
            lk1 = exp(log_lk1 + scale * ones(size(log_lk1)));
            %   p(H(n)|H(~n))
            like0 = pr0 * lk0;
            like1 = pr1 * lk1;
            like = like1 / (like0 + like1);
            u = rand(1,1);
            KAY(n,k) = le(u,like);
        end,
    end,
    if all(sum(KAY,1) < 3) == 1
        count = count + 1;
    else
        accept = 1;
    end,
    if count > 150
        KAY = draw.KAY;
        
        accept = 1;
    end,
end,
H = [zeros(n,1) ones(n,1) KAY];