%   This function does cross validation for the model in Hamilton and Owyang (2009)
%       The function outputs a score based on the entropy measure from the note
%       It requires R, the number of , and priors
function [score] = HO_cross_val(R,KAPPA)
score = 0;
[priors,results,draw] = HO_c_priors_CV(KAPPA);
draw_ = draw;

%*******************************************************************************************************
%   Partition the data into R pieces

% t_r(1) = 50;
% t_r(2) = 95;
% t_r(3) = 101;
% t_r(4) = 154;
nr = ceil(priors.T/R);
t_r = zeros(R-1,1);
t_r(1) = nr;
for r = 2:R-1
    t_r(r) = nr + t_r(r-1);
end,
%*******************************************************************************************************
%*******************************************************************************************************
%   compute the posterior distributions for the sample excluding the last kth
%*******************************************************************************************************
priors_ = priors;
%  estimation sample
priors_.YY = priors.YY(1:t_r(R-1),:);
%   validation sample
priors_.Y0 = priors.YY(t_r(R-1) + 1:end,:);
priors_.T = size(priors_.YY,1);
priors_.T_full = priors.T;
draw.Z = unidrnd(priors.K,priors_.T,1);
disp('K')
%*******************************************************************************************************
for n = 1:priors.Nkeep + priors.Nburn;
    %*******************************************************************************************************
    %    Draw the aggregate regime process Z conditional on H
    [draw.Z,draw.Sk] = HO_Z_CV(priors_,draw);
    %*******************************************************************************************************
    %    Draw the transition probabilities conditional on Z
    [draw.P] = HO_P_CV(priors_,draw);
    %*******************************************************************************************************
    %    Draw the parameters for the logit prior
    [draw.beta] = HO_MNL_CV(priors_,draw);
    %*******************************************************************************************************
    %    Draw the cluster
    [draw.H,draw.KAY,draw.beta,draw.lam,draw.z] = HO_H_CV(priors_,draw);
    %*******************************************************************************************************
    %    Draw mu0,mu1,sigsq for each state
    [draw.mu_0,draw.mu_1,draw.sigsq] = HO_THETA_CV(priors_,draw);
    %*******************************************************************************************************
    if n > priors.Nburn
        %   cross validation step
        %   make future ST
        [draw.Z_0,draw.Y_0] = HO_Z_cval_R(draw,priors_);
        %   compute the score for the generated ST_0
        [draw.score] = entropy(draw,priors_);
        %   compile the scores
        score = score + draw.score;
    end,
end,
%*******************************************************************************************************
%*******************************************************************************************************
%   compute the posterior distributions for the sample excluding the first kth
%*******************************************************************************************************
%*******************************************************************************************************
%   estimation sample
priors_.YY = priors.YY(t_r(1) + 1:end,:);
%   validation sample
priors_.Y0 = priors.YY(1:t_r(1),:);
priors_.T = size(priors_.YY,1);
priors_.T_full = priors.T;
draw = draw_;
draw.Z = unidrnd(priors.K,priors_.T,1);
%*******************************************************************************************************
%*******************************************************************************************************
disp('1')
for n = 1:priors.Nkeep + priors.Nburn;
    %*******************************************************************************************************
    %    Draw the aggregate regime process Z conditional on H
    [draw.Z,draw.Sk] = HO_Z_CV(priors_,draw);
    %*******************************************************************************************************
    %    Draw the transition probabilities conditional on Z
    [draw.P] = HO_P_CV(priors_,draw);
    %*******************************************************************************************************
    %    Draw the parameters for the logit prior
    [draw.beta] = HO_MNL_CV(priors_,draw);
    %*******************************************************************************************************
    %    Draw the cluster
    [draw.H,draw.KAY,draw.beta,draw.lam,draw.z] = HO_H_CV(priors_,draw);
    %*******************************************************************************************************
    %    Draw mu0,mu1,sigsq for each state
    [draw.mu_0,draw.mu_1,draw.sigsq] = HO_THETA_CV(priors_,draw);
    %*****************************************************************************************************
    if n > priors.Nburn
        %   cross validation step
        %   make future ST
        [draw.Z_0,draw.Y_0] = HO_Z_cval_1(draw,priors_);
        %   compute the score for the generated ST_0
        [draw.score] = entropy(draw,priors_);
        %   compile the scores
        score = score + draw.score;
    end,
end,
%*******************************************************************************************************
%*******************************************************************************************************
%   compute the posterior distributions for the sample excluding the middle kths
%*******************************************************************************************************
%*******************************************************************************************************
for r = 2:R-1
    disp(r)
    %   estimation sample
    priors_.YY = [priors.YY(1:t_r(r-1),:); priors.YY(t_r(r)+1:end,:)];
    %   validation sample
    priors_.Y0 = priors.YY(t_r(r-1) + 1:t_r(r),:);
    priors_.T = size(priors_.YY,1);
    priors_.T_full = priors.T;
    draw = draw_;
    draw.Z = unidrnd(priors.K,priors_.T,1);
    %*******************************************************************************************************
    %   compute the posterior distributions for the sample excluding the last kth
    for n = 1:priors.Nkeep + priors.Nburn;
        %*******************************************************************************************************
        %    Draw the aggregate regime process Z conditional on H
        [draw.Z,draw.Sk] = HO_Z_CV_rr(priors_,draw,t_r(r-1),t_r(r)-t_r(r-1));
        %*******************************************************************************************************
        %    Draw the transition probabilities conditional on Z
        [draw.P] = HO_P_CV_rr(priors_,draw,t_r(r-1));
        %*******************************************************************************************************
        %    Draw the parameters for the logit prior
        [draw.beta] = HO_MNL_CV(priors_,draw);
        %*******************************************************************************************************
        %    Draw the cluster
        [draw.H,draw.KAY,draw.beta,draw.lam,draw.z] = HO_H_CV(priors_,draw);
        %*******************************************************************************************************
        %    Draw mu0,mu1,sigsq for each state
        [draw.mu_0,draw.mu_1,draw.sigsq] = HO_THETA_CV(priors_,draw);
        %*****************************************************************************************************
        if n > priors.Nburn
            %   cross validation step
            %   make future ST
            [draw.Z_0,draw.Y_0] = HO_Z_cval_rr(draw,priors_,t_r,r);
            %   compute the score for the generated ST_0
            [draw.score] = entropy(draw,priors_);
            %   compile the scores
            score = score + draw.score;
        end,
    end,
end,